package LIDS.Server.Database;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import LIDS.Server.Exceptions.UserAlreadyExistsException;
import LIDS.Server.System.SessionManager;
import LIDS.Server.System.User;

public class DBAccessorTest {

	@Before
	public void setUp() throws Exception {
		DBAccessor.connect();
	}

	@After
	public void tearDown() throws Exception {
		removeDefaultUsers();
		DBAccessor.disconnect();
	}
	
	
	@Test
	public void addTest() { //Unnecessary since the session manager already incorporates this code into its test for adding users
			try{
				String username = "Bob Vila";
				String password = "password";
				User testUser = User.createUser(username, password, false);
				testUser = DBAccessor.add(testUser);
				User anotherUser = DBAccessor.retrieve(testUser.getName(),  "", false);
				assertTrue(testUser.equals(anotherUser));
				username = "";
				password = "emptyset";
				testUser = User.createUser(username, password, false);
				testUser = DBAccessor.add(testUser);
				assertTrue(testUser.equals((User.createUser("", "emptyset", false))));
				username = "emptyset";
				password = "";
				testUser = User.createUser(username, password, false);
				testUser = DBAccessor.add(testUser);
				assertTrue(testUser.equals(User.createUser("emptyset", "", false)));
				username = "";
				password = "";
				testUser = User.createUser(username, password, false);
				testUser = DBAccessor.add(testUser);
				fail();
			}catch(UserAlreadyExistsException ex){
				
			}
	}
	
	@Test
	public void removeTest(){
		addDefaultUsers();
		assertTrue(DBAccessor.isUserPresent("Bob Vila"));
		DBAccessor.remove("Bob Vila");
		assertFalse(DBAccessor.isUserPresent("Bob Vila"));
		assertTrue(DBAccessor.isUserPresent("emptyset"));
		DBAccessor.remove("emptyset");
		assertFalse(DBAccessor.isUserPresent("emptyset"));
	}
	
	@Test
	public void retrieve(){
		addDefaultUsers();
		User testUser = User.createUser("Bob Vila", "password", false);
		User retrievedUser = DBAccessor.retrieve(testUser.getName(),  "", false);
		assertTrue(testUser.equals(retrievedUser));
		testUser = User.createUser("", "emptyset", false);
		retrievedUser = DBAccessor.retrieve(testUser.getName(),  "", false);
		assertTrue(testUser.equals(retrievedUser));
		testUser = User.createUser("emptyset", "", false);
		retrievedUser = DBAccessor.retrieve(testUser.getName(),  "", false);
		assertTrue(testUser.equals(retrievedUser));
	}
	
	public void addDefaultUsers(){
		try{
			String username = "Bob Vila";
			String password = "password";
			User testUser = User.createUser(username, password, false);
			DBAccessor.add(testUser);
			username = "emptyset";
			password = "";
			testUser = User.createUser(username, password, false);
			DBAccessor.add(testUser);
		}catch(UserAlreadyExistsException ex){
			fail();
		}
	}
	
	public void removeDefaultUsers(){
		DBAccessor.remove("Bob Vila");
		DBAccessor.remove("emptyset");
	}
}
