package LIDS.Server.Database;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import LIDS.Server.Exceptions.UserAlreadyExistsException;
import LIDS.Server.System.SessionManager;
import LIDS.Server.System.User;

public class Database{
	
	private Connection db = null;
	private PreparedStatement insertStatement = null;
	private PreparedStatement provisionClearanceStatement = null;
	private PreparedStatement deleteStatement = null;
	private PreparedStatement retrieveStatement = null;
	private PreparedStatement retrieveStatement1 = null;
	
	public void connect(){
		try {
		   Class.forName("com.mysql.jdbc.Driver"); //must register the oracle driver for use by the jvm
		   try {
				db = DriverManager.getConnection("jdbc:mysql://:3306/accounts?autoreconnect=true&useSSL=false", "LIDS", "15b29ffdce66e10527a65bc6d71ad94d"); //creates db connection
				insertStatement = db.prepareStatement("Insert into users (username, password, clearanceLevel) "
						+ "VALUES(?, ?, ?)");
				provisionClearanceStatement = db.prepareStatement("UPDATE users SET clearanceLevel = ? where username = ?");
				deleteStatement = db.prepareStatement("DELETE FROM users WHERE username = ?");
				retrieveStatement = db.prepareStatement("Select username, password from users where username = ?");
				retrieveStatement1 = db.prepareStatement("Select username, password from users where username = ? and password = ?");
				}catch (SQLException e) {
				System.out.println("Fatal error " + e.getErrorCode() + " while attempting to connect to the mysql database and prepare statements");
				System.exit(1); //system is useless without the userdb
			}
		}catch(ClassNotFoundException ex) {
		   System.out.println("Error: unable to load driver class!");
		   System.exit(1);
		}
	}
		
	
	public User add(User aUser)throws UserAlreadyExistsException{ 
		if(isUserPresent(aUser.getName()) || aUser.equals(SessionManager.getNullUser())){
			throw new UserAlreadyExistsException();
		}
		try {
			this.getInsertStatement().setString(1, aUser.getName());
			this.getInsertStatement().setString(2, aUser.getHashedUserCreds());		
			this.getInsertStatement().setInt(3, aUser.getClearanceLevel());
			if(this.getInsertStatement().execute()){
				return aUser;
			}
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("Failed to set insertStaement variables");
		}
		return SessionManager.getNullUser();
	}
	
	public void provisionClearance(String aUsername, int newLevel){
		try {
			this.getProvisionClearanceStatement().setInt(1, newLevel);
			this.getProvisionClearanceStatement().setString(2, aUsername);
			this.getProvisionClearanceStatement().execute();
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("failed to update user clearance");
		}
	}
	
	public User remove(String aUserName){
		try {
			this.getDeleteStatement().setString(1, aUserName);
			this.getDeleteStatement().execute();
		} catch (SQLException e) {
			e.printStackTrace();
			System.out.println("failed to remove user");
		}
		return SessionManager.getNullUser();
	}
	
	public boolean isUserPresent(String aString){
		if(retrieve(aString, "", false).equals(SessionManager.getNullUser())){
			return false;
		}
		return true;
	}
	
	public User retrieve(String aUsername, String aPassword, boolean passCheck) {
		ResultSet result = null;
		if(aPassword == null || aPassword.isEmpty()){ 
			aPassword = new String(); 
			try {
				this.getRetrieveStatement().setString(1, aUsername);
				result = this.getRetrieveStatement().executeQuery();
			} catch (SQLException e) {
				e.printStackTrace();
				System.out.println("failed to set variables for the retireve statement");
			}
		}else{
			try {
				this.getRetrieveStatement1().setString(1, aUsername);
				this.getRetrieveStatement1().setString(2, aPassword);
				result = this.getRetrieveStatement1().executeQuery();
			} catch (SQLException e1) {
				e1.printStackTrace();
				System.out.println("failed to set variables for the retireve statement1");
				return SessionManager.getNullUser();
			}
		}
		try {
			if(result.next()){
				String username = result.getString("username");
				return User.createUser(username, (result.getString("password")), true);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return SessionManager.getNullUser();
	}
	
	public void disconnect(){
		try {
			db.close();
		} catch (SQLException e) {
			System.out.println("Failed to close database connection, possibly already closed.");
			e.printStackTrace();
		}
	}

	public List<User> retrieveAll() {
		ResultSet result = null;
		try{
			Statement sqlState = db.createStatement();
			result = sqlState.executeQuery("Select * from users;");
		} catch (SQLException e1) {
			e1.printStackTrace();
		}
		List<User> users = new ArrayList<User>();
		try {
			User tempUser = null;
			while(result.next()){
				tempUser = User.createUser(result.getString("username"), result.getString("password"), true);
				tempUser.setClearanceLevel(Integer.valueOf(result.getString("clearanceLevel")));
				users.add(tempUser);
			}
			return users;
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return null;
	}

	public PreparedStatement getInsertStatement() {
		return insertStatement;
	}

	public void setInsertStatement(PreparedStatement insertStatement) {
		this.insertStatement = insertStatement;
	}


	public PreparedStatement getProvisionClearanceStatement() {
		return provisionClearanceStatement;
	}


	public void setProvisionClearanceStatement(PreparedStatement provisionClearanceStatement) {
		this.provisionClearanceStatement = provisionClearanceStatement;
	}


	public PreparedStatement getDeleteStatement() {
		return deleteStatement;
	}


	public void setDeleteStatement(PreparedStatement deleteStatement) {
		this.deleteStatement = deleteStatement;
	}


	public PreparedStatement getRetrieveStatement() {
		return retrieveStatement;
	}


	public void setRetrieveStatement(PreparedStatement retrieveStatement) {
		this.retrieveStatement = retrieveStatement;
	}


	public PreparedStatement getRetrieveStatement1() {
		return retrieveStatement1;
	}


	public void setRetrieveStatement1(PreparedStatement retrieveStatement1) {
		this.retrieveStatement1 = retrieveStatement1;
	}
}
