package LIDS.Server.Database;

import static org.junit.Assert.*;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import LIDS.Server.Exceptions.UserAlreadyExistsException;
import LIDS.Server.System.SessionManager;
import LIDS.Server.System.User;

public class DatabaseTest {
	Database testDB = new Database();
	@Before
	public void setUp() throws Exception {
		testDB.connect();
	}

	@After
	public void tearDown() throws Exception {
		removeDefaultUsers();
		testDB.disconnect();
		testDB = null;
	}
	
	@Test
	public void addTest() {
		try{
			String username = "Bob Vila";
			String password = "password";
			User testUser = User.createUser(username, password, false);
			testUser = testDB.add(testUser);
			User anotherUser = testDB.retrieve(testUser.getName(), "", false);
			assertTrue(testUser.equals(anotherUser));
			username = "";
			password = "emptyset";
			testUser = User.createUser(username, password, false);
			testUser = testDB.add(testUser);
			assertTrue(testUser.equals((User.createUser("", "emptyset", false))));
			username = "emptyset";
			password = "";
			testUser = User.createUser(username, password, false);
			testUser = testDB.add(testUser);
			assertTrue(testUser.equals(User.createUser("emptyset", "", false)));
			username = "";
			password = "";
			testUser = User.createUser(username, password, false);
			testUser = testDB.add(testUser);
			fail();
		}catch(UserAlreadyExistsException ex){
			
		}
	}

	@Test
	public void removeTest(){
		testDB.remove("");
		assertTrue(testDB.retrieve("",  "", false).equals(SessionManager.getNullUser()));
		testDB.remove("Bob Vila");
		assertTrue(testDB.retrieve("Bob Vila",  "", false).equals(SessionManager.getNullUser()));
		testDB.remove("emptyset");
		assertTrue(testDB.retrieve("emptySet",  "", false).equals(SessionManager.getNullUser()));
	}
	
	@Test
	public void retrieveTest(){
		addDefaultUsers();
		User testUser = User.createUser("Bob Vila", "password", false);
		User retrievedUser = testDB.retrieve(testUser.getName(), "", false);
		assertTrue(testUser.equals(retrievedUser));
		testUser = User.createUser("", "emptyset", false);
		retrievedUser = testDB.retrieve(testUser.getName(), "", false);
		assertTrue(testUser.equals(retrievedUser));
		testUser = User.createUser("emptyset", "", false);
		retrievedUser = testDB.retrieve(testUser.getName(), "", false);
		assertTrue(testUser.equals(retrievedUser));
	}
	
	@Test
	public void getAllUsers(){
		addDefaultUsers();
		List<User> users = testDB.retrieveAll();
		assertTrue(users.size() == 3);
		removeDefaultUsers();
	}
	
	public void addDefaultUsers(){
		try{
			String username = "Bob Vila";
			String password = "password";
			User testUser = User.createUser(username, password, false);
			testDB.add(testUser);
			username = "emptyset";
			password = "";
			testUser = User.createUser(username, password, false);
			testDB.add(testUser);
		}catch(UserAlreadyExistsException ex){
			fail();
		}
	}
	
	public void removeDefaultUsers(){
		testDB.remove("Bob Vila");
		testDB.remove("emptyset");
	}
}
