package LIDS.Server.GUI;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.Dialog.ModalityType;

import java.util.ArrayList;
import java.util.Observable;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import LIDS.Server.System.LogHandler;
import LIDS.Server.System.NullUser;
import LIDS.Server.System.Sensor;
import LIDS.Server.System.HTTPSServerCom;
import LIDS.Server.System.SessionManager;
import LIDS.Server.System.SocketThreadHandler;

import java.util.Observer;

import net.miginfocom.swing.MigLayout;

public class DefaultPerspective extends Perspective implements Observer {

	private static final long serialVersionUID = -1434486780074125190L;
	private JPanel optionsPanel = null;
	private JPanel sensorDisplay = null;
	
	private GUIBase parent = null;
	
	private JTable table = null;

	private JLabel isArmed = null;
	private JLabel isTripped = null;
	
	public DefaultPerspective(GUIBase aBase){
		SocketThreadHandler.dataObserver(this);
		parent = aBase;
		optionsPanel = new JPanel();
		sensorDisplay = new JPanel();
		setupOptionsPanel(aBase);
		setupSensorDisplay(SocketThreadHandler.getSensorList());
		MigLayout layout = new MigLayout();
		this.setLayout(layout);
		add(optionsPanel);
		add(sensorDisplay, "gapleft 10mm, wrap 10mm");
		JLabel system = new JLabel("System Status: ");
		system.setFont(new Font("Calibri", Font.PLAIN, 35));
		JLabel alarm = new JLabel("Alarm Status: ");
		alarm.setFont(new Font("Calibri", Font.PLAIN, 35));
		add(system, "gapleft 10mm");
		add(alarm, "gapleft 10mm, wrap 5mm");
		if(!SocketThreadHandler.isArmed()){
			isArmed = new JLabel("Innactive");
			isArmed.setForeground(Color.BLUE);
		}else{
			isArmed = new JLabel("Batman is Watching");
			isArmed.setForeground(Color.RED);
		}
		isArmed.setFont(new Font("Times New Roman", Font.BOLD, 40));
		if(!SocketThreadHandler.isTripped()){
			isTripped = new JLabel("Clear");
			isTripped.setForeground(Color.BLUE);
		}else{
			isTripped = new JLabel("Intruder Detected");
			isTripped.setForeground(Color.RED);
		}
		isTripped.setFont(new Font("Times New Roman", Font.BOLD, 40));
		add(isArmed, "gapLeft 15mm");
		add(isTripped, "gapleft 20mm");
		SocketThreadHandler.addMeAsSensorObserver(this);
	}

	private static WindowListener closeWindow = new WindowAdapter(){
			public void windowClosingEvent(WindowEvent e){
				e.getWindow().dispose();
			}
	};
	
	public void toggleArmStatus(){
		if(SocketThreadHandler.isArmed()){
			isArmed.setText("Batman is Watching");
			isArmed.setForeground(Color.RED);
		}else{
			isArmed.setText("Innactive");
			isArmed.setForeground(Color.BLUE);
		}
		isTripped.setText("Clear");
		isTripped.setForeground(Color.BLUE);
	}
	
	public void setTripStatus(){
		isTripped.setText("Intruder Detected");
		
	}
	
	private void setupOptionsPanel(final GUIBase aBase) {
		optionsPanel.setLayout(new MigLayout("", "[center]10mm", "[center]5mm"));
		optionsPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		JComponent label = new JLabel("System Commands");
		optionsPanel.add(label, "wrap 50, ");
		JComponent tba = new JButton("Arm / Disarm");
		((JButton) tba).addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event){
				SocketThreadHandler.armSystem();
				toggleArmStatus();
			}
		});
		optionsPanel.add(tba, "gapleft 5mm");
		tba = new JButton("Add User");
		((JButton) tba).addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event){
				parent.updateUI(new ProvisionUserPerspective(parent));
			}
		});
		optionsPanel.add(tba, "wrap 50");
		tba = new JButton("Shutdown / Logout");
		((JButton) tba).addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event){
				//TODO trigger system shutdown and the emission of a sleep signal
				final JDialog diag = new JDialog(aBase);
				diag.setModalityType(ModalityType.APPLICATION_MODAL);
				diag.addWindowListener(closeWindow);
				JButton shut = new JButton("Shutdown");
				shut.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent event) {
						System.exit(0);
					}
				});
				JButton log = new JButton("Logout");
				log.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						SessionManager.setUser(new NullUser());
						aBase.updateUI(new NullPerspective());
						diag.dispose();
					}
					
				});
				JButton cancel = new JButton("Cancel");
				cancel.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent arg0) {
						diag.dispose();
					}
				});
				diag.setLayout(new MigLayout());
				diag.add(shut, "cell 1 0, gapleft 8mm");
				diag.add(log, "cell 3 0, gapleft 5mm, wrap 5mm");
				diag.add(cancel, "cell 3 5, gapleft 5mm");
				diag.setBounds(275, 275, 300, 150);
				diag.setVisible(true);
			}
		});
		optionsPanel.add(tba, "gapleft 5mm");
		tba = new JButton("Provision User Clearance");
		((JButton) tba).addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event){
				parent.updateUI(new ClearancePerspective(parent));
			}
		});
		optionsPanel.add(tba, "wrap 50");
		tba = new JButton("Access Stored Logs");
		((JButton) tba).addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event){
				parent.updateUI(new LogPerspective(parent, LogHandler.importLogFile(), WIDTH, HEIGHT));
			}
		});
		optionsPanel.add(tba, "gapleft 5mm");
		tba = new JButton("Check For Updates");
		((JButton) tba).addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event){
				//TODO implements downloadable updates
			}
		});
		optionsPanel.add(tba, "wrap 50");
	}
	
	public void setupSensorDisplay(ArrayList<Sensor> sensors){
		sensorDisplay.setLayout(new MigLayout());
		String columnNames[] = {"ID", "Armed", "Alarm"};
		NonEditableDefaultTableModel sensorModel = new NonEditableDefaultTableModel(columnNames, sensors.size() + 1);
		table = new JTable(sensorModel);
		table.setFillsViewportHeight(true);
		if(sensors.size() >= 1){
			for(int i = 0; i < sensors.size(); i++){
				table.setValueAt(sensors.get(i).getID(), i, 0);
				table.setValueAt(sensors.get(i).isArmed(), i, 1);
				table.setValueAt(sensors.get(i).isAlarmStatus(), i, 2);
				//table.setValueAt(sensors.get(i).getPower(), i, 3);
			}
		}
		JScrollPane displayPane = new JScrollPane(table);
		sensorDisplay.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		sensorDisplay.setMaximumSize(new Dimension(800, 323));
		sensorDisplay.add(displayPane);
		
	}

	public void changeSystemText(){
		if(!SocketThreadHandler.isArmed()){
			getIsArmed().setText("Innactive"); 
			getIsArmed().setForeground(Color.BLUE);
		}else{
			getIsArmed().setText("Batman is Watching");
			getIsArmed().setForeground(Color.RED);
		}
		getIsArmed().setFont(new Font("Times New Roman", Font.BOLD, 40));
		if(!SocketThreadHandler.isTripped()){
			getIsTripped().setText("Clear");
			getIsTripped().setForeground(Color.BLUE);
		}else{
			getIsTripped().setText("Intruder Detected");
			getIsTripped().setForeground(Color.RED);
		}
		getIsTripped().setFont(new Font("Times New Roman", Font.BOLD, 40));
	}
	
	private JLabel getIsArmed() {
		return isArmed;
	}
	
	private JLabel getIsTripped() {
		return isTripped;
	}
	
	public void modifySensorDisplay(){
		ArrayList<Sensor> sensors = SocketThreadHandler.getSensorList();
		table = null;
		sensorDisplay.removeAll();
		setupSensorDisplay(sensors);
		sensorDisplay.setVisible(false);
		sensorDisplay.setVisible(true);
	}
	
	public void update(Observable o, Object arg) {
		modifySensorDisplay(); //doesn't display updates until clicked on
		setChanged();		
	}
}
