package LIDS.Server.NetworkEvents.Test;

import LIDS.Server.NetworkEvents.EventTable;
import LIDS.Server.NetworkEvents.FileWrapper;
import LIDS.Server.NetworkEvents.NetworkEvent;
import LIDS.Server.NetworkEvents.RequestManager;
import LIDS.Server.NetworkEvents.Scanner;
import LIDS.Server.System.SessionManager;
import LIDS.Server.System.User;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class NetworkEventsTest
{
  public NetworkEventsTest() {}
  
  private FileWrapper aFileNode = null;
  
  @Before
  public void setUp() throws Exception {
    aFileNode = new FileWrapper("");
    EventTable.initializeNetworkTable();
    SessionManager.init();
  }
  
  @org.junit.After
  public void tearDown() throws Exception {
    aFileNode = null;
  }
  
  @Test
  public void ExecuteNumberRequestTest() {
    String aJsonRequest = "{\"username\":\"Admin\"}";
    String aSecondJsonRequest = "{\"username\":\"nonAdmin\"}";
    int testInt1 = 0;
    int testInt2 = 0;
    int testInt3 = 0;
    EventTable.getTable().getEvent("numReq").execute(aJsonRequest, getTestFileNode(), null);
    testInt1 = Integer.valueOf(getTestFileNode().getProccessResults()).intValue();
    setTestFileNode(new FileWrapper("sdfgsdfg"));
    EventTable.getTable().getEvent("numReq").execute(aSecondJsonRequest, getTestFileNode(), null);
    testInt2 = Integer.valueOf(getTestFileNode().getProccessResults()).intValue();
    setTestFileNode(new FileWrapper("sdfdfghdfghfhgsdfg"));
    EventTable.getTable().getEvent("numReq").execute(aJsonRequest, getTestFileNode(), null);
    testInt3 = Integer.valueOf(getTestFileNode().getProccessResults()).intValue();
    Assert.assertTrue(testInt1 >= 0);
    Assert.assertTrue(testInt2 < 0);
    Assert.assertTrue(testInt1 != testInt3);
  }
  
  @Test
  public void SessionLoginTest() {
    EventTable.getTable().getEvent("numReq").execute("{\"username\":\"Admin\"}", getTestFileNode(), null);
    String theNumber = getTestFileNode().getProccessResults();
    String theBasePassword = User.hashUserCreds("swordfish");
    String thePassword = User.hashUserCreds(theNumber.concat(theBasePassword));
    String aJsonRequest = "{\"username\":\"Admin\",\"password\":\"" + thePassword + "\"}";
    String aFailedJsonRequest = "{\"username\":\"Admin\",\"password\":\"swordfish\"}";
    EventTable.getTable().getEvent("processLogin").execute(aJsonRequest, aFileNode, null);
    Assert.assertTrue(getTestFileNode().getProccessResults().equals("pub/main.html"));
    EventTable.getTable().getEvent("processLogin").execute(aFailedJsonRequest, aFileNode, null);
    Assert.assertFalse(getTestFileNode().getProccessResults().equals("pub/main.html"));
  }
  
  @Test
  public void testRequestManager() {
    String request = "POST / HTTP/1.1\n\tHost: 127.0.0.1\nConnection: keep-alive\n Cache-Control: max-age=0\nUpgrade-Insecure-Requests: 1\n\tUser-Agent: Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36\nAccept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8\nDNT: 1\n Accept-Encoding: gzip, deflate, sdch,br\n Accept-Language: en-US,en;q=0.8,id;q=0.6";
    Scanner theScanner = new Scanner();
    theScanner.scanString(request);
    RequestManager theManager = (RequestManager)EventTable.getTable().getEvent("newRequest");
    theManager.setScanner(theScanner);
    theManager.execute("", null, null);
  }
  
  public FileWrapper getTestFileNode()
  {
    return aFileNode;
  }
  
  public void setTestFileNode(FileWrapper testFileNode) {
    aFileNode = testFileNode;
  }
}