package LIDS.Server.System;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

public class DataRelay extends Observable implements Observer
{
  private ServerSocket serverSocket = null;
  
  private static ArrayList<Sensor> sensorList;
  private int indexCounter = 0;
  private int sensorCount = 0;
  
  private boolean isSystemArmed = false;
  private boolean alarmTripped = false;
  
  private BufferedOutputStream socketWriter = null;
  private BufferedInputStream socketReader = null;
  

  public DataRelay() {
    sensorList = new ArrayList<Sensor>();
    openServerSocket();
  }
  
  public ArrayList<Sensor> getSensors() {
    return sensorList;
  }
  
  public void listenForConnections() {
    try {
      Socket readSocket = null;
      readSocket = serverSocket.accept();
      socketWriter = new BufferedOutputStream(readSocket.getOutputStream());
      socketReader = new BufferedInputStream(readSocket.getInputStream());
      Sensor aSensor = new Sensor(readSocket, socketWriter, socketReader);
      System.out.println("Sensor connected");
      while (aSensor.getID() == -59999) {
        Thread.sleep(10L);
      }
      SensorManager.addSensor(aSensor);
      System.out.println("Sensor count: " + String.valueOf(SensorManager.getSensorCount()));
    } catch (IOException ex) {
	      System.out.println("Failed to open a socket");
	}catch (InterruptedException e) {
      e.printStackTrace();
    }
  }
  
  public void addSensor(Sensor aSensor){
	  int index = contains(aSensor);
	  if(index < 0){
	      getSensors().add(aSensor);
	      aSensor.mayIWatch(this);
	      lazyNotify();
	      getSensors().sort(null);
	      System.out.println("Sensor added id = " + aSensor.getID());
	  }else{
		  System.out.println("Sensor Replaced");
		  getSensors().get(index).closeStreams();
		  getSensors().remove(index);
		  getSensors().add(aSensor);
	  }
  }
    
  private int contains(Sensor aSensor){
    return containsHelper(aSensor.getID(), 0, getSensors().size() - 1);
  }
  
  private int containsHelper(int anID, int low, int high) {
    if (low == high) {
      return (getSensors().get(low).getID() == anID ? low : -1);
    }
    if (low < 0 || high < 0) return -1;
    if (getSensors().get(low).getID() == anID){return low;}
    if (getSensors().get(high).getID() == anID) {return high;}
    int mid = (int)Math.floor(low + high / 2);
    return (mid > anID) ? containsHelper(anID, low, mid - 1) : containsHelper(anID, mid, high);
  }
  
  public void updateScreen()
  {
    lazyNotify();
  }
  
  public void lazyNotify() {
    setChanged();
    notifyObservers(this);
  }
  
  public void openServerSocket() {
    try {
      serverSocket = new ServerSocket(7644);
      serverSocket.setReuseAddress(true);
    } catch (java.net.BindException ex) {
      System.out.println("Failed to bind the socket to the specified port;");
    } catch (IOException ex) {
      System.out.println("Failed to open the server socket for listening.");
    }
  }
  
  public static void write(BufferedWriter socketWriter, String aMessage) {
    try {
      socketWriter.write(aMessage.toString() + '\r' + '\n');
      socketWriter.flush();
    } catch (IOException e) {
      System.out.println("Failed to write possible socket closure");
    }
  }
  
  public int getIndex() {
    return indexCounter;
  }
  
  public synchronized void closeConnections() {
    for (Sensor sense : sensorList) {
      sense.closeStreams();
    }
    sensorList = null;
    lazyNotify();
  }
  
  public void armSystem() {
    System.out.println(sensorList.size());
    for (Sensor eachSensor : sensorList) {
      eachSensor.arm();
      setTripped();
      lazyNotify();
    }
  }
  
  public void removeMe(Sensor networkedModule) {
	  try{
		  sensorList.remove(networkedModule);
		  updateScreen();  
	  }catch(NullPointerException ex){
		  return;
	  }
  }
  
  public void addObserverToSensors(Observer anOberver) {
    for (Sensor eachSensor : sensorList) {
      eachSensor.mayIWatch(anOberver);
    }
  }
  
  public void calculateRemoteDisconnections() {
    for (Sensor eachSensor : getSensors()) {
      if (!eachSensor.checkSensorConnection()) {
        removeMe(eachSensor);
      }
    }
    lazyNotify();
  }
  
  public void setTripped() {
    alarmTripped = true;
    notifyObservers();
  }
  
  public void alertUser() {
    alarmTripped = true;
    System.out.println("ALERT");
    lazyNotify();
  }
  
  public boolean isArmed() {
	boolean atLeastOne = false;
    for(Sensor aSensor : this.getSensors()){
    	atLeastOne = atLeastOne || aSensor.isArmed();
    }
    if(atLeastOne){
    	for(Sensor aSensor : this.getSensors()){
        	aSensor.setArmed(true);
        }
	}
    return atLeastOne;
  }
  
  public boolean isTripped() {
	  boolean atLeastOne = false;
	    for(Sensor aSensor : this.getSensors()){
	    	atLeastOne = atLeastOne || aSensor.isAlarmStatus();
	    }
	    return atLeastOne;
  }
  
  public void mayIWatch(Observer anObserver) {
    addObserver(anObserver);
    System.out.println(anObserver.getClass().toString().concat(" is watching ".concat(getClass().toString())));
  }
  
  public void update(Observable arg0, Object arg1)
  {
    calculateRemoteDisconnections();
    lazyNotify();
  }
}