package LIDS.Server.System;

import LIDS.Server.NetworkEvents.RequestManager;
import LIDS.Server.NetworkEvents.Scanner;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.Writer;
import java.util.Date;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;

public class HTTPSServerThread
  extends Thread
{
  private SSLSocket sslSocket = null;
  
  public HTTPSServerThread(SSLSocket sslSocket) {
    this.sslSocket = sslSocket;
  }
  
  public void run() {
    sslSocket.setEnabledCipherSuites(sslSocket.getSupportedCipherSuites());
    
    try
    {
      sslSocket.startHandshake();
      

      SSLSession sslSession = sslSocket.getSession();
      
      BufferedInputStream aBufferedSocketInputStream = new BufferedInputStream(sslSocket.getInputStream());
      BufferedOutputStream aBufferedSocketOutputStream = new BufferedOutputStream(sslSocket.getOutputStream());
      
      Scanner anHttpMessageScanner = new Scanner(aBufferedSocketInputStream);
      new RequestManager(anHttpMessageScanner, aBufferedSocketOutputStream).execute("", null, aBufferedSocketOutputStream);
      sslSocket.close();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
  

































  private boolean sendFile(String filePath, BufferedOutputStream aBufferedSocketOutputStream)
  {
    boolean socketBroken = false;
    BufferedInputStream aBufferedFileStream = null;
    try {
      System.out.println(filePath);
      File theFile = new File(filePath);
      aBufferedFileStream = new BufferedInputStream(new FileInputStream(theFile));
      sendMetaData(aBufferedSocketOutputStream, theFile);
      while (aBufferedFileStream.available() > 0) {
        byte[] temp = new byte['È'];
        aBufferedFileStream.read(temp, 0, 200);
        System.out.println(temp);
        aBufferedSocketOutputStream.write(temp);
      }
      aBufferedSocketOutputStream.write("\\r\\n".getBytes());
      aBufferedSocketOutputStream.flush();
      aBufferedFileStream.close();
    }
    catch (IOException e) {
      socketBroken = true;
      e.printStackTrace();
    }
    return socketBroken;
  }
  
  private void sendMetaData(BufferedOutputStream aBufferedSocketOutputStream, File theFile)
  {
    try {
      aBufferedSocketOutputStream.write("HTTP/1.0 200\r\n".getBytes());
      System.out.println("Date:".concat(new Date().toString()));
      aBufferedSocketOutputStream.write("Date:".concat(new Date().toString().concat("\r\n")).getBytes());
      aBufferedSocketOutputStream.write("Server: Java SE 7\r\nAccepted-Ranges: bytes\r\nConnection: close\r\n".getBytes());
      


      String contentType = getContentType(theFile);
      
      System.out.println("Content-type: ".concat(contentType.concat("\r\n")));
      aBufferedSocketOutputStream.write("Content-type: ".concat(contentType.concat("\r\n\r\n")).getBytes());
      return;
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }
  
  private String getContentType(File theFile)
  {
    String[] pieces = theFile.getName().split("\\.");
    String str = pieces[(pieces.length - 1)]; 
    switch (str) {
    	case "js":  
    		return "aaplication/json";
    	case "css":  
    		return "text/css"; 
    	case "ico":  
    		return "image/x-icon"; 
    	case "jpg":  
    		return "image/jpeg"; 
    	case "html":  
    		return "text/html";
    }
    return "text/plain";
  }
  
  private void reportError(Writer aWriteStream) {
    try {
      aWriteStream.write("HTTP/1.1 500");
      aWriteStream.flush();
    } catch (IOException e1) {
      System.out.println("Failed to report server error to client");
      e1.printStackTrace();
    }
  }
  
  private void closeStream(BufferedReader aFileReader) {
    try {
      aFileReader.close();
    } catch (IOException e) {
      System.out.println("Failed to close fileReadStream");
      e.printStackTrace();
    }
  }
}