package LIDS.Server.System;

import LIDS.Server.NetworkEvents.EventTable;
import LIDS.Server.NetworkEvents.JsonWrapper;
import LIDS.Server.NetworkEvents.NetworkEvent;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.StringReader;
import java.net.Socket;
import java.nio.charset.StandardCharsets;
import java.util.Observer;

import javax.json.Json;
import javax.json.JsonObject;
import javax.json.JsonObjectBuilder;
import javax.json.JsonReader;

public class Sensor extends java.util.Observable implements Runnable, Comparable
{
  private boolean armed = false;
  private boolean alarmStatus = false;
  private boolean hasIdBeenSet = false;
  
  private double power = 101.1D;
  
  private int ID = -59999;
  
  private BufferedInputStream socketReader = null;
  private BufferedOutputStream socketWriter = null;
  
  private Socket associatedSocket = null;
  
  public Sensor(Socket aSocket, BufferedOutputStream aSocketWriter, BufferedInputStream aSocketReader) {
    associatedSocket = aSocket;
    socketWriter = aSocketWriter;
    socketReader = aSocketReader;
    Thread sensorThread = new Thread(this);
    sensorThread.start();
    LogHandler.exportToLogFile("Sensor " + ID + " connected");
  }
  

  public void recieveData(NetworkedModule aModule, String data) {}
  

  public void run()
  {
	
    String stringBuffer = "";
    try {
    	while (associatedSocket.isConnected()){ 
    		byte[] buffer = new byte[' '];
    		int bytesRead = 8192;
    		if ((!associatedSocket.isConnected())){ break; }
    		System.out.println("Started");
    		while ((bytesRead == 8192)) {
    			System.out.println("read:");
    			bytesRead = socketReader.read(buffer);
            	stringBuffer = stringBuffer + new String(buffer, StandardCharsets.US_ASCII);
            	System.out.println(" " + stringBuffer);
            	System.out.println(stringBuffer);
    		}
        	JsonReader jsonReader = Json.createReader(new StringReader(stringBuffer));
    		
    		JsonObject jsonObject = jsonReader.readObject();
    		System.out.println("I got : " + jsonObject.getString("command"));
    		EventTable.getTable().getEvent(jsonObject.getString("command"))
    			.execute(jsonObject, this, getSocketWriter());
    		jsonReader.close();
    		setChanged();
    		notifyObservers(this);
        } 
      return;
    }
    catch (IOException e) {
      SocketThreadHandler.removeMe(this);
      lazyNotify();
    }
  }
  

  public boolean checkSensorConnection()
  {
    return (!associatedSocket.isClosed()) && (associatedSocket.isConnected());
  }
  
  public boolean isArmed() {
    return armed;
  }
  
  public void arm() {
    JsonObject jObject = JsonWrapper.getObjectBuilder()
      .add("responseCode", 201).add("isArmed", !isArmed()).build();
    EventTable.getTable().getEvent("armSystem").execute(jObject, this, getSocketWriter());
    setArmed(!isArmed());
  }
  
  public boolean isAlarmStatus()
  {
    return alarmStatus;
  }
  
  public void setAlarmStatus(boolean newAlarmStatus) {
    alarmStatus = newAlarmStatus;
  }
  
  public double getPower() {
    return power;
  }
  
  public void setPower(double power) {
    this.power = power;
  }
  
  public void setSocketReader(BufferedInputStream socketReader) {
    this.socketReader = socketReader;
  }
  
  public BufferedOutputStream getSocketWriter() {
    return socketWriter;
  }
  
  public void setSocketWriter(BufferedOutputStream socketWriter) {
    this.socketWriter = socketWriter;
  }
  
  public Socket getAssociatedSocket() {
    return associatedSocket;
  }
  
  public void setAssociatedSocket(Socket associatedSocket) {
    this.associatedSocket = associatedSocket;
  }
  
  public BufferedInputStream getSocketReader() {
    return socketReader;
  }
  
  public int getID() {
    return ID;
  }
  
  public void setID(int iD) {
    ID = iD;
    hasIdBeenSet = true;
  }
  
  public void mayIWatch(Observer anObserver) {
    addObserver(anObserver);
    System.out.println(anObserver.getClass().toString().concat(getClass().toString()));
  }
  
  public void closeStreams() {
    try {
      getSocketReader().close();
      getSocketWriter().close();
      getAssociatedSocket().close();
    }
    catch (IOException localIOException) {}
  }
  
  public boolean getHasIdBeenSet()
  {
    return hasIdBeenSet;
  }
  
  public void lazyNotify() {
    setChanged();
    notifyObservers(this);
  }
  
  public void setArmed(boolean newStatus) {
    armed = newStatus;
    if(!armed){ setAlarmStatus(false); }
  }
  
  public int compareTo(Object arg0)
  {
    Sensor aSensor = (Sensor)arg0;
    return getID() == aSensor.getID() ? 0 : getID() > aSensor.getID() ? 1 : -1;
  }
  
  @Override
  public String toString(){
	  return JsonWrapper.getObjectBuilder()
			  .add("isArmed", String.valueOf(this.isArmed()))
			  .add("alarmStatus", this.isAlarmStatus())
			  .add("ID", this.getID()).build().toString();
  }
}