package LIDS.Server.System;

import LIDS.Server.GUI.GUIBase;

import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Observable;
import LIDS.Server.Database.DBAccessor;
import LIDS.Server.Exceptions.InvalidUserException;
import LIDS.Server.Exceptions.UserAlreadyExistsException;

public class SessionManager extends Observable{//This class is a coordinator and Controller therefore does not 
											   //actually do any work besides sending messages and setting variables
											   //and because of which does not need a testclass 
	private static User currentUser = null;
	private static User nullUser = new NullUser();
	private static GUIBase window = null;
	private static String errorBuffer = null;
	private static HashMap<String, User> netUsers = null;
	private static HashMap<String, User> activeSessions = null;
	private static SecureRandom numberGenerator = null;
	
	public static void init(){
		numberGenerator = new SecureRandom(String.valueOf(System.currentTimeMillis()).getBytes());
		activeSessions = new HashMap<String, User>();
		LogHandler.init();
		DBAccessor.connect();
		currentUser = getNullUser();
		netUsers = new HashMap<String, User>();
		window = new GUIBase();
	}
	
	public static boolean addActiveSession(String providedSessionID, String providedPassword,
			String sentNumber, User aNetUser){
		String number1 = String.valueOf(sentNumber);
		String calculatedPassword = User.hashUserCreds(number1.concat(aNetUser.getHashedUserCreds()));
		String calculatedSessionID = User.hashUserCreds(aNetUser.getName().concat(String.valueOf(aNetUser.getNumber())));
		if(calculatedPassword.equals(providedPassword) && providedSessionID.equals(calculatedSessionID)){
			aNetUser.setSessionID(calculatedSessionID);
			activeSessions.put(aNetUser.getName(), aNetUser);
			netUsers.remove(aNetUser.getName());
			return true;
		}
		return false;
	}
	
	public static boolean holdUser(String aUserName){
		User aNetUser = DBAccessor.retrieve(aUserName, "", false);
		if(!aNetUser.equals(SessionManager.getNullUser())){
			netUsers.put(aUserName, aNetUser);
			return true;
		}
		return false;
	}
	
	public static User getHeldUser(String username){
		User aUser = netUsers.get(username);
		if(aUser == null){ 
			SessionManager.holdUser(username); 
			aUser = netUsers.get(username);
		}
		return (aUser == null)? getNullUser() : aUser;
	}
	
	public static boolean isNullUser(User aUser){
		return (aUser.equals(getNullUser()))? true : false;
	}
	
	public static User addUser(String username, String password){
		//TODO 0) user 1) super user 2) technician 3) senior technician 4) Admin 5) Master Admin
		
		User newUser = User.createUser(username, password, false);
		try {
			User.checkEmptyUsername(newUser);
			newUser = DBAccessor.add(newUser);
			LogHandler.exportToLogFile("Added the User " + username);
			return newUser;
		} catch (UserAlreadyExistsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidUserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return getNullUser();
	}
	
	public static User removeUser(String aUserName){
		if(currentUser.getClearanceLevel() < 1){ return new NullUser(); }
		LogHandler.exportToLogFile("Removed the User " + aUserName);
		return DBAccessor.remove(aUserName);
	}
	
	public static User removeUser(User aUser){
		if(currentUser.getClearanceLevel() < 1){ return new NullUser(); }
		return DBAccessor.remove(aUser.getName());
	}
	
	public static boolean isUserPresent(String aUsername){
		return DBAccessor.isUserPresent(aUsername);
	}
	
	public static void setUser(User aUser){
		currentUser = aUser;
	}
	
	public static User getCurrentUser(){
		return currentUser;
	}

	public static boolean login(String username, String password, boolean updateGUI) {
		User tempUser = User.createUser(username, password, false);
		tempUser = DBAccessor.retrieve(username, tempUser.getHashedUserCreds(), true);
		if(updateGUI){
			currentUser = tempUser;
			window.updateUI(tempUser);
		}
		LogHandler.exportToLogFile(currentUser.getName() + " Logged in");
		return (currentUser.getClearanceLevel() != -1);
	}
	
	public static void setErrorBuffer(String aString){
		errorBuffer = aString;
	}	
	
	public static User getNullUser(){
		return nullUser;
	}

	public static void logout() {
		setUser(getNullUser());
	}
	
	public static SecureRandom getNumberGenerator() {
		return numberGenerator;
	}

	public static boolean authenticateSession(String sessionID, String sessionUsername){
		User aUser = activeSessions.get(sessionUsername);
		boolean temp = !aUser.getSessionID().equals(sessionID);
		//if(aUser == null || temp){
		if(temp){
			return false;
		}else{
			return true;
		}
		
	}
	
	public static void setNumberGenerator(SecureRandom numberGenerator) {
		numberGenerator = numberGenerator;
	}
}
