package LIDS.Server.System;

import static org.junit.Assert.*;

import org.junit.Test;

import LIDS.Server.Database.DBAccessor;
import LIDS.Server.Exceptions.UserAlreadyExistsException;

import org.junit.After;
import org.junit.Before;

public class SessionManagerTest {
	
	@Before
	public void setUp(){
		SessionManager.init();
		User testUser = User.createUser("Admin", "swordfish", false);
		testUser.setClearanceLevel(5);
		SessionManager.setUser(testUser);
	}
	
	@Test
	public void addUserTest(){
		String username = "Bob Vila";
		String password = "password";
		User testUser = SessionManager.addUser(username, password);
		assertTrue(testUser.getHashedUserCreds().equals("126680608771750945340162210354335764377"));
		//username = "";
		//password = "emptyset";
		//testUser = SessionManager.addUser(username, password);
		//assertTrue(testUser.getHashedUserCreds().equals("dd132b9389a335959b11bb7710547311"));
		username = "emptyset";
		password = "";
		testUser = SessionManager.addUser(username, password);
		assertTrue(testUser.getHashedUserCreds().equals("-1"));
		username = "";
		password = "";
		testUser = SessionManager.addUser(username, password);
		assertTrue(testUser.getHashedUserCreds().equals(""));
		removeDefaultUsers();
	}
	
	@Test
	public void removeTest(){
		addDefaultUsers();
		SessionManager.removeUser("Bob Vila");
		assertFalse(SessionManager.isUserPresent("Bob Vila"));
		SessionManager.removeUser("emptyset");
		assertFalse(SessionManager.isUserPresent("emptyset"));
		
	}
	
	@Test
	public void loginTest(){
		SessionManager.init();
		addDefaultUsers();
		SessionManager.login("someName", "somePassword", true);
		removeDefaultUsers();
	}
	
	public void addDefaultUsers(){
		try{
			String username = "Bob Vila";
			String password = "password";
			User testUser = User.createUser(username, password, false);
			DBAccessor.add(testUser);
			username = "emptyset";
			password = "";
			testUser = User.createUser(username, password, false);
			DBAccessor.add(testUser);
		}catch(UserAlreadyExistsException ex){
			fail();
		}
	}
	
	@Test
	public void updateUITest(){
		fail("Not yet implemented");
	}
	
	@After
	public void tearDown(){
		SessionManager.logout();
		LogHandler.destruct();
		DBAccessor.disconnect();
	}
	
	public void removeDefaultUsers(){
		DBAccessor.remove("Bob Vila");
		DBAccessor.remove("emptyset");
	}
}